# Crane
#
# Sample Webots controller file for driving the
# underactuated 1-2 pendulum.  The robot has a driven
# base joint between the base and link1, and passive
# joints between link1 and the two distal links link2A
# and link2B.
#
# No copyright, 2020-2021, Garth Zeglin.  This file is
# explicitly placed in the public domain.

print("loading crane.py...")

# Import the Webots simulator API.
from controller import Robot
from controller import Keyboard

# Import the standard Python math library.
import math, random, time

print("crane.py waking up...")

# Define the time step in milliseconds between
# controller updates.
EVENT_LOOP_DT = 20

# Request a proxy object representing the robot to
# control.
robot = Robot()
name = robot.getName()
keyboard = Keyboard()
keyboard.enable(EVENT_LOOP_DT)

# Attempt to randomize the random library sequence.
random.seed(time.time())

# Initialize geometric constants.  These should match
# the current geometry of the robot.
wheel_radius = 0.1
axle_length  = 0.14

if name == 'Wobbly':
    # Fetch handles for the wheel motors
    l_motor = robot.getDevice('left wheel motor')
    r_motor = robot.getDevice('right wheel motor')
    
    # Adjust the low-level controller gains.
    print("%s: setting PID gains." % (name))
    l_motor.setControlPID(1.0, 0.0, 0.1)
    r_motor.setControlPID(1.0, 0.0, 0.1)
    
    # Fetch handles for the wheel joint sensors.
    l_pos_sensor = robot.getDevice('left wheel sensor')
    r_pos_sensor = robot.getDevice('right wheel sensor')
    
    # Specify the sampling rate for the joint sensors.
    l_pos_sensor.enable(EVENT_LOOP_DT)
    r_pos_sensor.enable(EVENT_LOOP_DT)
    
    # Connect to the accelerometer sensor.
    accelerometer = robot.getDevice('accelerometer')
    accelerometer_timer = 0
    accelerometer_interval = 50
    accelerometer.enable(accelerometer_interval)
    accel_vector = [0, 0, 9.81] # reference pose value
    

elif name == 'Crane':
    # Fetch handles for the joint sensors.
    j1  = robot.getDevice('baseJoint')
    j2 = robot.getDevice('midJoint')
    
    # Specify the sampling rate for the joint sensors.
    j1.enable(EVENT_LOOP_DT)
    j2.enable(EVENT_LOOP_DT)
    
    # Fetch handle for the 'base' joint motor.  In this
    # example the motor will be controlled as a torque
    # motor, bypassing the lower-level PID control.
    motor1 = robot.getDevice('baseMotor')
    motor1.setPosition(0.0)
    
    
    motor2 = robot.getDevice('midMotor')
    motor2.setPosition(0)
    #motor2.setTorque(2)
    
    # initial position of motors
    q1 = 0
    q2 = 0
    keyReleased = False

################################################################
# Define movement commands
            
def go_forward(velocity):
    """Command the motor to turn at the rate which produce the ground velocity
       specified in meters/sec.  Negative values turn backward. """

    # velocity control mode
    l_motor.setPosition(math.inf)
    r_motor.setPosition(math.inf)

    # calculate the rotational rate in radians/sec based on the wheel radius
    theta_dot = velocity / wheel_radius
    l_motor.setVelocity(theta_dot)
    r_motor.setVelocity(theta_dot)
    return
    
def go_rotate(rot_velocity):
    """Command the motors to turn in place at the rate which produce the rotational
       velocity specified in radians/sec.  Negative values turn
       backward."""

    # velocity control mode
    l_motor.setPosition(math.inf)
    r_motor.setPosition(math.inf)

    # calculate the difference in linear velocity of the wheels
    linear_velocity = axle_length * rot_velocity

    # calculate the net rotational rate in radians/sec based on the wheel radius
    theta_dot = linear_velocity / wheel_radius

    # apply the result symmetrically to the wheels
    l_motor.setVelocity( 0.5*theta_dot)
    r_motor.setVelocity(-0.5*theta_dot)
    return
    
def go_still():
    """Actively damp any wobble to come to rest in place."""
    # map an error in X acceleration to a linear velocity
    vel = -0.05 * accel_vector[0]
    go_forward(vel)
    # print("go_still: %f, %f" % (self.accel_vector[0], vel))
    return

################################################################


# Run an event loop until the simulation quits,
# indicated by the step function returning -1.
while robot.step(EVENT_LOOP_DT) != -1:

    # Read simulator clock time.
    t = robot.getTime()
    
    key = keyboard.getKey()

    # Read any computer keyboard keypresses.  Returns -1 or an integer keycode while a key is held down.
    if name == 'Crane':
        if key != -1:
        
            # set torques
            key = chr(key).lower()
            
            if key == 'a':
                motor1.setTorque(-5)
            elif key == 'd':
                motor1.setTorque(5)
            elif key == 'w':
                motor2.setTorque(10)
            elif key == 's':
                motor2.setTorque(-5)
            elif key == 'e':
                motor2.setPosition(0)
            elif key == 'r':
                motor1.setPosition(0)
                
            
                
            keyReleased = True  
    else:
        if key != -1:
            # convert the integer key number to a lowercase single-character string,
            # then set the target pose indicated by the key
            letter = chr(key).lower()
            if letter == 'i':
                go_forward(1.0)
            elif letter == 'j':
                go_rotate(-1.0)
            elif letter == 'l':
                go_rotate(1.0)
            elif letter == 'k':
                go_forward(0.0)
                go_rotate(0.0)
            elif letter == '.':
                go_forward(-1.0)
            else:
                go_still()